
% preprocess images from 
% The German Traffic Sign Benchmark (GTSRB)
% http://benchmark.ini.rub.de/?section=gtsrb&subsection=dataset
%
% this took 5247 seconds for 51839 images in Octave 4.4.1 on my notebook
% and could certainly be optimized a lot nore...

% ----------------------------------------------------------------------------------------
% preprocess the annotated training images (GTSRB_Final_Training_Images.zip and GTSRB_Final_Test_GT.zip)
% https://sid.erda.dk/public/archives/daaeac0d7ce1152aea9b61d9f1e19370/GTSRB_Final_Training_Images.zip
% https://sid.erda.dk/public/archives/daaeac0d7ce1152aea9b61d9f1e19370/GTSRB_Final_Test_GT.zip
%
% training images - total: 39209, good: 39209
% ----------------------------------------------------------------------------------------

clear all;

cnt_total = 0;
cnt_good = 0;

n = 1;

% 43 classes

for cid = 0 : 42

    fprintf('class: %d\n', cid);
    dir = sprintf('GTSRB_Final_Training_Images/GTSRB/Final_Training/Images/%05d', cid);
    
    file = fopen(sprintf('%s/GT-%05d.csv', dir, cid), 'r');
    fgetl(file); % skip header
    tab = textscan(file, '%s %d %d %d %d %d %d %d', 'Delimiter', ';');
    fclose(file);
    
    for k = 1:length(tab{1})
        
        cnt_total = cnt_total + 1;
        
        % skip corrupted images (none for Octave 4.4.1)
        try 
            img = imread(sprintf('%s/%s', dir, tab{1}{k}));
        catch
            continue;
        end
        
        % ROI (region of interest) is x1,y1,x2,y2
        w  = tab{2}(k);
        h  = tab{3}(k);
        x1 = tab{4}(k);
        y1 = tab{5}(k);
        x2 = tab{6}(k);
        y2 = tab{7}(k);

        % make the ROI quadratic
        square = min([ x2 - x1, y2 - y1 ]);
        while x2 - x1 > square
            if x2 - x1 > square
                x2 = x2 - 1;
            end
            if x2 - x1 > square
                x1 = x1 + 1;
            end
        end
        while y2 - y1 > square
            if y2 - y1 > square
                y2 = y2 - 1;
            end
            if y2 - y1 > square
                y1 = y1 + 1;
            end
        end
        
        %  ROI consistency check 
        if x1 < 1 || x2 < 1 || x1 > w || x2 > w
            error('ROI-x consistency check failed');
        end
        if y1 < 1 || y2 < 1 || y1 > h || y2 > h
            error('ROI-y consistency check failed');
        end
        
        % extract ROI and convert 8 bit to single in range 0 - 1
        img = single(img(y1:y2,x1:x2, :)) / 255;
        
        % consistency check
        if size(img, 1) ~= square + 1  || size(img, 2) ~= square + 1
            error('ROI size consistency check failed');
        end
        
        % convert to grayscale and scale to 30 x 30 pixel
        img = mean(img, 3);
        img = imresize(img, [30 30] );
        
        % stretch dynamic to  0.01 .. 0.99
        lower = min(min(img));
        upper = max(max(img));
        img = 0.01 + (img - lower) / (upper - lower) * 0.98;
        
        % dynamic consistency check
        if min(min(img)) < 0.01 - 0.001 || max(max(img)) > 0.99 + 0.001
            error('image dynamic consistency check failed');
        end
        
        % store the image as a row in a matrix, first the image class, then the pixels
        gtsrb_train(n, :) = [ single(cid) reshape(img', [1 30 * 30]) ];
        n = n + 1;
        
        % show image
        % imshow(imresize(img, [300 300]));
        % pause(0.01);
        
        cnt_good = cnt_good + 1;

    end
end

% store the matrix with all the images 

fprintf('training images - total: %d, good: %d\n', cnt_total, cnt_good);
save('-v7', 'gtsrb_train.mat', 'gtsrb_train');


% ----------------------------------------------------------------------------------------
% preprocess the test images (GTSRB_Final_Test_Images.zip )
% https://sid.erda.dk/public/archives/daaeac0d7ce1152aea9b61d9f1e19370/GTSRB_Final_Test_Images.zip
%
% testing images - total: 12630, good: 12630
% ----------------------------------------------------------------------------------------

clear all;

cnt_total = 0;
cnt_good = 0;

n = 1;

dir = 'GTSRB_Final_Test_Images/GTSRB/Final_Test/Images';
    
file = fopen('GTSRB_Final_Test_GT/GT-final_test.csv', 'r');
fgetl(file); % skip header
tab = textscan(file, '%s %d %d %d %d %d %d %d', 'Delimiter', ';');
fclose(file);
    
for k = 1:length(tab{1})
        
    cnt_total = cnt_total + 1;

    % skip corrupted images (none for Octave 4.4.1)
    try 
        img = imread(sprintf('%s/%s', dir, tab{1}{k}));
    catch
        continue;
    end

    % ROI (region of interest) is x1,y1,x2,y2
    w  = tab{2}(k);
    h  = tab{3}(k);
    x1 = tab{4}(k);
    y1 = tab{5}(k);
    x2 = tab{6}(k);
    y2 = tab{7}(k);
    cid = tab{8}(k);

    % make the ROI quadratic
    square = min([ x2 - x1, y2 - y1 ]);
    while x2 - x1 > square
        if x2 - x1 > square
            x2 = x2 - 1;
        end
        if x2 - x1 > square
            x1 = x1 + 1;
        end
    end
    while y2 - y1 > square
        if y2 - y1 > square
            y2 = y2 - 1;
        end
        if y2 - y1 > square
            y1 = y1 + 1;
        end
    end

    %  ROI consistency check 
    if x1 < 1 || x2 < 1 || x1 > w || x2 > w
        error('ROI-x consistency check failed');
    end
    if y1 < 1 || y2 < 1 || y1 > h || y2 > h
        error('ROI-y consistency check failed');
    end

    % extract ROI and convert 8 bit to single in range 0 - 1
    img = single(img(y1:y2,x1:x2, :)) / 255;

    % consistency check
    if size(img, 1) ~= square + 1  || size(img, 2) ~= square + 1
        error('ROI size consistency check failed');
    end

    % convert to grayscale and scale to 30 x 30 pixel
    img = mean(img, 3);
    img = imresize(img, [30 30] );

    % stretch dynamic to  0.01 .. 0.99
    lower = min(min(img));
    upper = max(max(img));
    img = 0.01 + (img - lower) / (upper - lower) * 0.98;

    % dynamic consistency check
    if min(min(img)) < 0.01 - 0.001 || max(max(img)) > 0.99 + 0.001
        error('image dynamic consistency check failed');
    end

    % store the image as a row in a matrix, first the image class, then the pixels
    gtsrb_test(n, :) = [ single(cid) reshape(img', [1 30 * 30]) ];
    n = n + 1;

    % show image
    % imshow(imresize(img, [300 300]));
    % pause(0.01);

    cnt_good = cnt_good + 1;

end

% store the matrix with all the images 
fprintf('testing images - total: %d, good: %d\n', cnt_total, cnt_good);
save('-v7', 'gtsrb_test.mat', 'gtsrb_test');

