% train and test the neural network (nn) for the
% The German Traffic Sign Benchmark (GTSRB)
% http://benchmark.ini.rub.de/?section=gtsrb&subsection=dataset
%
% nn_init_net.m, nn_query_net.m, nn_train_net.m and nn_sigmoid.m
% is a very simple implementation of a fully connected neural network
% with an input layer, one hidden layer and an output ("loss") layer,
% the math is taken from the book
% "Make Your Own Neural Network" by Tariq Rashid - there is also a
% german translation ("Neuronale Netze selbst programmieren")
% 
% this took 508 seconds for training for 5 epochs in Octave 4.4.1 on my notebook;
% Octave nicely used all 8 cores as it does matrix operations with OpenBLAS
%
% applying the net to all 12630 test images took just 3 seconds
% correct detection rate is ~ 85 %
% 
% training more than 5 epochs does not improve detection rate significantly
%
% the net is saved in trained_net.mat


% -------------------------------------------------------------------------
% define the nn
%   30*30 nodes in the input layer 
%   1000 nodes in the hidden layer 
%   43 nodes in the output layer
%   learning rate = 0.04
net = nn_init_net(30 * 30, 1000, 43, 0.04);

% -------------------------------------------------------------------------
% train the nn with data from gtsrb_train
% (39209 lines, each containing the class index and 900 pixel values)
load('gtsrb_train.mat');
fprintf('training:\n');
tic;
for epoch = 1:5
  % as the rows are sorted by class, we shuffle them at each epoch
  ix = randperm(length(gtsrb_train));
  gtsrb_train = gtsrb_train(ix, :);
  for k = 1:length(gtsrb_train)
    inputs = gtsrb_train(k, 2:end);
    responses = zeros(43,1) + 0.01;
    responses(gtsrb_train(k, 1) + 1) = 0.99;
    net = nn_train_net(net, inputs, responses);
    if mod(k, 5000) == 0
      fprintf('%d ', k);
    end
  end
  fprintf('%d \n', k);
end
toc;

% -------------------------------------------------------------------------
% test the performance of the trained net with the test data from gtsrb_test
% (12630 lines, each containing the class index and 900 pixel values)
load('gtsrb_test.mat');
fprintf('testing:\n');
tic;
good_count = 0;
for k = 1:length(gtsrb_test)
  estimate = nn_query_net(net, gtsrb_test(k, 2:end));
  [val, pos] = max(estimate);
  if gtsrb_test(k, 1) == pos - 1
    good_count = good_count + 1;
  end
  if mod(k, 5000) == 0
    fprintf('%d ', k);
  end
end
fprintf('%d \n', k);
toc;

% for 5 epochs this gets ~ 85% correct
fprintf('correct %.2f\n', 100.0 * good_count / length(gtsrb_test));

% save the net
save('-v7', 'trained_net.mat', 'net');

